@@ -7,6 +7,7 @@ from models import BulkList
 from urllib import urlencode
 from tornado.ioloop import IOLoop
 from tornado.httpclient import AsyncHTTPClient, HTTPRequest
+from tornado.concurrent import return_future
 
 
 class ESConnection(object):
@@ -37,6 +38,7 @@ class ESConnection(object):
         }
         return path
 
+    @return_future
     def search(self, callback, **kwargs):
         path = self.create_path("search", **kwargs)
         source = json.dumps(kwargs.get('source', {"query": {"query_string": {"query": "*"}}}))
@@ -45,6 +47,7 @@ class ESConnection(object):
     def multi_search(self, index, source):
         self.bulk.add(index, source)
 
+    @return_future
     def apply_search(self, callback):
         path = "/_msearch"
         source = self.bulk.prepare_search()
@@ -55,14 +58,56 @@ class ESConnection(object):
         request_http = HTTPRequest(url, method="POST", body=source)
         self.client.fetch(request=request_http, callback=callback)
 
+    @return_future
     def get_by_path(self, path, callback):
         url = '%(url)s%(path)s' % {"url": self.url, "path": path}
         self.client.fetch(url, callback)
 
+    @return_future
     def get(self, index, type, uid, callback):
         def to_dict_callback(response):
             source = json.loads(response.body).get('_source', {})
             callback(source)
+        self.request_document(index, type, uid, callback=to_dict_callback)
+
+    @return_future
+    def put(self, index, type, uid, contents, parameters=None, callback=None):
+        self.request_document(
+            index, type, uid, "PUT", body=json.dumps(contents),
+            parameters=parameters, callback=callback)
+
+    @return_future
+    def delete(self, index, type, uid, parameters=None, callback=None):
+        self.request_document(index, type, uid, "DELETE", parameters=parameters, callback=callback)
+
+    @return_future
+    def count(self, index="_all", type=None, source='', parameters=None, callback=None):
+        path = '/{}'.format(index)
+
+        if type:
+            path += '/{}'.format(type)
+
+        path += '/_count'
+
+        if parameters:
+            path += '?{}'.format(urlencode(parameters or {}))
+
+        if source:
+            source = json.dumps(source['query'])
+
+        self.post_by_path(path=path, callback=callback, source=source)
+
+    def request_document(self, index, type, uid, method="GET", body=None, parameters=None, callback=None):
         path = '/{index}/{type}/{uid}'.format(**locals())
-        url = '%(url)s%(path)s' % {"url": self.url, "path": path}
-        self.client.fetch(url, to_dict_callback)
+        url = '%(url)s%(path)s?%(querystring)s' % {
+            "url": self.url,
+            "path": path,
+            "querystring": urlencode(parameters or {})
+        }
+        request_arguments = dict(method=method)
+
+        if body is not None:
+            request_arguments['body'] = body
+
+        request = HTTPRequest(url, **request_arguments)
+        self.client.fetch(request, callback)
